
/*---------------------------------------------
		
	MatNN library : Test App 1 (cpp)

		* This is not an terrain poly pushing demo.

		This shows a silly brainwave of the neural
		net. The nets are totally untrained (random weights).

		The neural net (in this example) has 3 input neurons
		that take the x / y position of a 2d grid and the time/ticks
		returned by GetTickCount(); It has 3 outputs = RGB color which
		is also used as a height.
		 
		We/me/i graph the response of the the net over a small 2d plane
		of x/y values, at a specific time (using GetTickCount() ) to create
		a pretty swirly jaggy heightmap thingy (tm)

		keys : r = random neural net
			   a = decrease response
			   d = increase response

	By Matt Barnett

		(see References)	* See matnn.h for more information
	
---------------------------------------------*/

//basic glut and math stuff for camera etc
 
#include <ctime>
#include <cstdio>
#include <windows.h>
#include <GL/glut.h>

#include "../../../imath/imath.h"
#include "../../../matnn/matnn.h"

GLfloat grey[] = {0.2,0.2,0.2,1.0};
GLfloat above_origin[] = {0.0,0.0,3.0,1.0};
GLfloat white[] = {1.0,1.0,1.0,1.0};
GLfloat	shiny[] = {50.0};

#define WINDOW_SIZE			1024		//
#define	SCALE_MOTION		100			//scale down by

int x,y,mx,my,xs,ys;


int oldmx = 0;
int oldmy = 0;

enum modetype { MODE_ROTATE,MODE_MOVE,MODE_CAM };
modetype mode;
iquaternion4 camquat;

//globals for testing here

using MatLayeredNeuralNetLib::NeuralNet;
float resp = 0.45f;
float inputs[3];
float outputs[3];


//change the number of hidden layers
//(number of inputs, number of outputs, number of neurons in hidden layers, number of hidden layers)
NeuralNet n1(3,3,4,3,inputs,outputs);

 
//------------------------
	
void drawaxes()
{
  glDisable(GL_LIGHTING);

  glBegin(GL_LINES);
  glColor3f(1.0,0.0,0.0);glVertex3f(-0.5,0.0,0.0);glVertex3f(1.2,0.0,0.0);
  glColor3f(0.0,1.0,0.0);glVertex3f(0.0,-0.5,0.0);glVertex3f(0.0,1.2,0.0);
  glColor3f(0.0,0.0,1.0);glVertex3f(0.0,0.0,-0.5);glVertex3f(0.0,0.0,1.2);
  glEnd();
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	imatrix16 rm(camquat);
	glTranslatef(0.0,0.0,-0.8);
	glMultMatrixf(rm.m);
	
	//instert drawing here
	
	float floorsize = 1.5;

	//if you want more tri-strips then decrease this value
	float floorstep = 0.025f;

	float i,j;

	float fftime = (float)GetTickCount() / 1500;

	glPushMatrix();
	glTranslatef((float)-floorsize/2,(float)-floorsize/2,0.0);

	for (i=0;i<floorsize;i+=floorstep)
	{
		glBegin(GL_TRIANGLE_STRIP);
		for (j=0;j<floorsize;j+=floorstep)
		{
			
				inputs[0] = j;
				inputs[1] = i;
				inputs[2] = fftime;	
				n1.RunNN();

				glColor3f(outputs[0],outputs[1],outputs[2]);			//rem this out and try one of the below
				//glVertex3f(i,j,outputs[2] / 9.0 )				
				//glVertex3f(i,j,0.0f);
				glVertex3f(i,j,(outputs[0]+outputs[1]+outputs[2]) / 35.0f);

				inputs[0] = j+floorstep;
				inputs[1] = i+floorstep;
				inputs[2] = fftime;	
				n1.RunNN();

				glColor3f(outputs[0],outputs[1],outputs[2]);			//rem this out and try one of the below (chose the same as above)
				//glVertex3f(i+floorstep,j+floorstep,outputs[2] / 9.0 );
				//glVertex3f(i+floorstep,j+floorstep,0.0f);
				glVertex3f(i+floorstep,j+floorstep,(outputs[0]+outputs[1]+outputs[2])  / 35.0f);

		}
		glEnd();
	}


	drawaxes();
	glPopMatrix();


	glPopMatrix();
	glutSwapBuffers();
	glutPostRedisplay();
}



void reshape(int w, int h)
{
  glViewport(0,0,(GLsizei) w,(GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(90,(float)w/h,0.01,100.0);
}



void motion(int x, int y)
{
	mx = x;my = y;

	xs = mx - oldmx;
	ys = oldmy - my;

	oldmx = mx;
	oldmy = my;

	iquaternion4 multquat;

	if (mode == MODE_ROTATE)
	{	
	
	}
	else if (mode == MODE_MOVE)
	{

	}
	else if (mode == MODE_CAM)
	{
		multquat.makeEulerRotation( ivector3( (float)ys/SCALE_MOTION,-(float)xs/SCALE_MOTION,0.0 ) );
		camquat.multiply(multquat);
	}
}


void mouse(int button,int state,int x,int y)
{
	oldmx = x;
	oldmy = y;

	if (button == GLUT_LEFT_BUTTON)
		mode = MODE_ROTATE;

	if (button == GLUT_RIGHT_BUTTON)
		mode = MODE_MOVE;

	
	if (button == GLUT_MIDDLE_BUTTON)
		mode = MODE_CAM;
}

void init(void)
{

	srand( (unsigned)time( NULL ) );	


	glEnable(GL_DEPTH_TEST);
	//glEnable(GL_LIGHTING);
	//glEnable(GL_LIGHT0);
	//glLightfv(GL_LIGHT0,GL_POSITION,above_origin);
	
	//glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	//glMaterialfv(GL_FRONT,GL_DIFFUSE,grey);
	//glMaterialfv(GL_FRONT,GL_SPECULAR,white);
	//glMaterialfv(GL_FRONT,GL_SHININESS,shiny);

	//glEnable(GL_POLYGON_OFFSET_FILL);
	//glPolygonOffset(0.1,0.1);
	//glDepthFunc(GL_LEQUAL);
	//glEnable(GL_DEPTH_TEST);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	//glEnable(GL_BLEND);
	//glEnable(GL_POLYGON_SMOOTH);
	//glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	glPointSize(2.0f);
	

	inputs[0] = 0;
	inputs[1] = 0;

	n1.SetResponse(resp);
}

void key(GLubyte key, int x, int y)
{
  switch (key)
    {
			case 'a':resp -= 0.05f;printf("response %f\n",resp);n1.SetResponse(resp);
			break;

			case 'd':resp += 0.05f;printf("response %f\n",resp);n1.SetResponse(resp);
			
			break;

			case 'r':im_random_seed_time();n1.Randomize();

			break;

			case '1':

			break;

			case '!':

			break;


			case 'l':

			break;


			case 'q':exit(0);
			break;
    }
	
}

int main(int argc, char **argv)
{

	meInitMemoryCheck();
	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInitWindowSize(WINDOW_SIZE,WINDOW_SIZE);
	glutInitWindowPosition(100,10);
	glutCreateWindow("Matts test");

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	glutMotionFunc(motion);
	glutMouseFunc(mouse);

	init();
	glutMainLoop();

	return 0;
}