/*---------------------------------------------
		
	Inline Math library 
	(imath , im_)

	By Matt Barnett
		(see References)

	-----------
	portability
	-----------

		exclusivly uses the internal float datatype throughout
		and precision and portability are dependent on that.

	------------
	classes
	------------
		
		ivector3
		ibasis9			
		imatrix16		
		iquaternion4
		iray6
		iplane4
		ibox3
		

  	----------------
	other components
	----------------

		* float constants collection

		* lookup table class (for functions such as sin/cos/tan/acos etc)
		* random number lookup table class

		* interfaces to generalise common math functions
		
			im_random_num()			//uses lookup tables
			im_random_clamp()
			im_sin()				//uses lookup tables
			im_cos()				//uses lookup tables
			im_acos()
			im_tan()				//uses lookup tables
			im_sqrt()			


  	-------------------------------
	external components [optional]
	-------------------------------
		
		ibox3
		(closly intergrated with imath)

	------------
	class design
	------------

		imath.h					//include this file
		imath.cpp				//add this file to project
		imath.lib?				//or link to this?

		this file, imath.h sould include the files

		imath_vmq.h				//vector mathrix and quats all together
		imath_trace.h			//rays and planes together
		imath_box.h				//bounding boxes
		imath_lut.h				//lookup tables

		* All functions are inlined (the i stands for inline) ( this should be optimal, ie: the compilier chooses )

		* Operators are generally not implemented using member functions calls.  This is deliberate.  Member function
		  calls quite specifically operate on an object, and operators act between them.
		
		* Integral types are passed/returned by value
		  math objects are passed/returned by reference where appropriate

		* All members are public (for core classes). Classes are
		  designed to mimic integral types(hmm, dont that too seriously).

	-----------
	References:		( INCOMPLETE )
	-----------
		
		 1. PI constants from windows calculator :)
			Allthough you cant store such an accurate PI very easily
			its worth noting that wincalc will give pi to 21 decimal places.

		 2. Game Programming Gems 2
		 3. Elemetry Linear Algebra	  
		 4. Real2Int function By Sree (* See imath_real2int.h )
		 5. www.gametutorials.com	  (specifically the quaternion example, great site)
		 6. matrix & quaternion FAQ	  (everything about m's and q's)
		 7. Effective C++ 2e		  (great book)
		 8. flipcode http://www.flipcode.com/cgi-bin/msg.cgi?showThread=00002476&forum=general&id=-1
		    (angle between 2 vectors thread) Paul Hope's angle code
	
---------------------------------------------*/



//-------- headers etc -------
//---------------------------- 

#ifndef _IMATH_MAIN_H_HEADER_069389570
#define _IMATH_MAIN_H_HEADER_069389570

#include "../leakdetect.h"

#include <iostream>

using std::cout;
using std::endl;

#include <math.h>						//fpu maths
#include <cassert>						//assert
#include <windows.h>					//opengl needs windows (comment for non WIN32 platforms)
#include <gl/gl.h>						//opengl for drawing vectors etc (minimal, not important)

#include "../myerror.h"

//----------------------------
//----------------------------



//----------------------------
//----------------------------

//alias/type/macro your favorites
#define		RANDOM_NUM							im_random_num()
#define		RANDOM_CLAMP						im_random_clamp()

//macroed settings
#define		_IVECTOR3_VERBOSE_NORMALISE_FAIL_

//----------------------------
//----------------------------



//----------------------------
//----------------------------

class ivector3;
class float_lookuptable;

//----------------------------
//----------------------------



//---- constants -------------------
//
// see imath.cpp for
// constant definitions and comments
//----------------------------------

extern const float  cnf_epsilon;			//"small" value (considered 0)
extern const float			 pi;			
extern const float		half_pi;			
extern const float		 two_pi;
extern const float _pi_over_180;
extern const float _180_over_pi;

//extern const ivector3 xaxis;
//extern const ivector3 yaxis;
//extern const ivector3 zaxis;
//extern const ivector3 zero;

//extern const ivector3 red;  
//extern const ivector3 green;
//extern const ivector3 blue;
//extern const ivector3 black;

//extern const ivector3 cyan;
//extern const ivector3 magenta;
//extern const ivector3 yellow;

extern const float_lookuptable sintable;
extern const float_lookuptable costable;
extern const float_lookuptable tantable;

//----------------------------
//----------------------------



//-- imath component headers --
//-----------------------------

#include "imath_real2int.h"		//need first by lut
#include "imath_lut.h"			//lookup tables
#include "imath_core.h"			//trig, sqrt,  etc

#include "imath_vector.h"		//vector is needed by trace
#include "imath_trace.h"		//trace is needed by matrix
#include "imath_mq.h"			//matrix and quaternion

#include "imath_box.h"			//bounding boxess

//---------------------------- 
//---------------------------- 

#endif