/*---------------------------------------------
		
	MatNN library : Implementation (cpp)

		* See Header file for more information

	By Matt Barnett
		(see References)
	
---------------------------------------------*/

#include "matnn.h"

//----------------------------
//----------------------------
using MatLayeredNeuralNetLib::Neuron;
using MatLayeredNeuralNetLib::NeuronLayer;
using MatLayeredNeuralNetLib::NeuralNet;
using MatLayeredNeuralNetLib::TrainableNeuralNet;

//the constructors look a bit nasty because a reference to
//the response variable/constant is passed down by the NeuralNet
//owner class (this allows one copy per NeuralNet , and when it changes
//there is no need to update each neuron/layer copy)

//
//	MatNN::Neuron
//

//do not copy neurons
//Neuron::Neuron(Neuron &copy)
//{	 
//}
//Neuron &Neuron::operator=(Neuron &rhs)
//{
//
//	return *this;
//}


Neuron::Neuron(int in_inputs,float &in_sig_response) : sig_response(in_sig_response)
{
	num_inputs = in_inputs;

	//inputs = 0;
	//inputs = meNew Neuron * [num_inputs];
	
	weights = 0;
	weights = meNew float [num_inputs];

	RandomizeWeights();
}

void Neuron::RandomizeWeights()
{
	//randomize each weight 0.0f - 1.0f
	for (int i=0;i<num_inputs;i++)
	{
		weights[i] = RANDOM_CLAMP;
	}
}

void Neuron::Mutate(float chance,float extent)
{
	//mutate one of my weights by +/- extent
	//int i = RANDOM_NUM * num_inputs;
	//weights[i] += RANDOM_CLAMP * extent;
	
	//mutate any/some/all neuron weights
	for (int i=0;i<num_inputs;i++)
		if (RANDOM_NUM < chance) weights[i] += RANDOM_CLAMP * extent;
}

float Neuron::Calc(float *inputs)
{
	float out = 0;
	for (int x=0;x<num_inputs;x++)
	{
		out += weights[x] * inputs[x];
	
	}

	//sigmoid reponse curve
	//return ( 1 / (sig_response + (float)exp( - (out / 10) )) );
	
	//sinus response curve
	return im_sin( out / sig_response);
}

Neuron::~Neuron()
{
	//cleanup
	meDelete [] weights;weights = 0;
}

//
//	MatNN::NeuronLayer
//

NeuronLayer::NeuronLayer(NeuronLayer &copy)
{
	num_neurons = copy.num_neurons;

	//cannot be assigned yet!!
	//this must be fixed by the caller
	input = 0;

	output = 0;
	output = meNew float [num_neurons];
	neurons = 0;
	neurons = meNew Neuron * [num_neurons];	
	
	//same number of neurons as previous layer
	for (int i=0;i<num_neurons;i++)
	{
		//create neurons
		neurons[i] = 0;	
		neurons[i] = meNew Neuron(copy.neurons[0]->num_inputs,sig_response);		//one float input for each neuron in the environmental/input layer
	
		//copy over the weights from copy
		memcpy( neurons[i]->weights,copy.neurons[i]->weights,sizeof(float) * copy.neurons[0]->num_inputs );
	}
}

//No =
//NeuronLayer &NeuronLayer::operator=(NeuronLayer &rhs)
//{
//
//	return *this;
//}

NeuronLayer::NeuronLayer(int in_neurons,int in_num_inputs,float *finputs)
{
	num_neurons = in_neurons;

	input = 0;
	input = finputs;

	output = 0;
	output = meNew float [num_neurons];
	neurons = 0;
	neurons = meNew Neuron * [num_neurons];	
	
	for (int i=0;i<num_neurons;i++)
	{
		neurons[i] = 0;	
		neurons[i] = meNew Neuron(in_num_inputs,sig_response);		//one float input for each neuron in the environmental/input layer
	}
	
	if (!input)
	{
		printf("NeuronLayer::NeuronLayer() no input array specified!\n");
		exit(0);
	}	
}

NeuronLayer::~NeuronLayer()
{	
	meDelete [] output;
	output = 0;

	for (int i=0;i<num_neurons;i++)
	{
		meDelete neurons[i];
		neurons[i] = 0;	
	}
	
	meDelete [] neurons;
	neurons = 0;
}	


void NeuronLayer::RunLayer()
{
	for (int x = 0;x<num_neurons;x++)
	{
		output[x] = neurons[x]->Calc(input);
	}
}

void NeuronLayer::Randomize()
{
	for (int x = 0;x<num_neurons;x++)
		neurons[x]->RandomizeWeights();
} 

int NeuronLayer::Mutate(float chance,float extent)
{
	int mutated = 0;

	//if chance, then mutate 
	for (int x = 0;x<num_neurons;x++)
	{
		if (RANDOM_NUM < chance)
		{
			neurons[x]->Mutate(chance,extent);
			mutated++;
		}
	}

	return mutated;
} 

//
//	MatNN::NeuralNet
//

NeuralNet::NeuralNet(int in_neurons,int out_neurons,int hidden_neurons,int hidden_layers,float *in_finput,float *in_foutput)
{
	Sleep(30);
	im_random_seed_time();

	//sanity checks first
	if (num_hidden_layers < 0 || num_hidden_layers > MATNN_MAX_HIDDEN_LAYERS)
	{
		cMyError e;e.FatalError("hidden_layers must be  0 => < = %i!",MATNN_MAX_HIDDEN_LAYERS);
		exit(EXIT_FAILURE);
	}

	if (in_neurons < 0 || in_neurons > MATNN_MAX_IN_NEURONS)
	{
		cMyError e;e.FatalError("in_neurons must be  0 => < = %i!",MATNN_MAX_IN_NEURONS);
		exit(EXIT_FAILURE);
	}

	if (out_neurons < 0 || out_neurons > MATNN_MAX_OUT_NEURONS)
	{
		cMyError e;e.FatalError("out_neurons must be  0 => < = %i!",MATNN_MAX_OUT_NEURONS);
		exit(EXIT_FAILURE);
	}

	if (hidden_neurons < 0 || hidden_neurons > MATNN_MAX_HIDDEN_NEURONS)
	{
		cMyError e;e.FatalError("hidden_neurons must be  0 => < = %i!",MATNN_MAX_HIDDEN_NEURONS);
		exit(EXIT_FAILURE);
	}


	//get input & output pointers
	input = in_finput;
	output = in_foutput;

	//grap those parameters
	num_input_neurons = in_neurons;
	num_output_neurons = out_neurons;
	num_hidden_neurons = hidden_neurons;
	num_hidden_layers = hidden_layers;
	
	//null em
	input_layer = 0;		
	output_layer = 0;
	parray_hidden_layers = 0;

	//create input layer
	input_layer = meNew NeuronLayer(num_input_neurons,num_input_neurons,input);		

	//an array of pointers to hidden layers	
	if (num_hidden_layers) 
	{
		parray_hidden_layers = meNew NeuronLayer * [num_hidden_layers];
		if (!parray_hidden_layers)
		{
			cMyError e;e.FatalError("Could not allocate %i hidden layers!",num_hidden_layers);
			
			meDelete input_layer;

			exit(EXIT_FAILURE);
		}
	}

	//null each one and create it
	for (int i=0;i<num_hidden_layers;i++)
	{
		parray_hidden_layers[i] = 0;

		if (i == 0)
			parray_hidden_layers[i] = meNew NeuronLayer(num_hidden_neurons, input_layer->num_neurons,input_layer->output);
		else
			parray_hidden_layers[i] = meNew NeuronLayer(num_hidden_neurons, parray_hidden_layers[i - 1]->num_neurons , parray_hidden_layers[i - 1]->output);
	}

	//output layer attached to last hidden layer
	//or to input layer (special case)
	if (num_hidden_layers == 0)
		output_layer = meNew NeuronLayer(num_output_neurons , input_layer->num_neurons , input_layer->output);
	else
		output_layer = meNew NeuronLayer(num_output_neurons , parray_hidden_layers[num_hidden_layers - 1]->num_neurons , parray_hidden_layers[num_hidden_layers - 1]->output);



	//anything reasonable is ok here
	//as long as the net has been constructed
	SetResponse(1.0f);

}

//No =
//NeuralNet &NeuralNet::operator=(NeuralNet &rhs)
//{
//	return *this;
//}

NeuralNet::NeuralNet(NeuralNet &copy)
{
	//starting with the simple stuff
	
	sig_response = copy.sig_response;
	num_input_neurons = copy.num_input_neurons;
	num_output_neurons = copy.num_input_neurons;
	num_hidden_neurons = copy.num_hidden_neurons;
	num_hidden_layers = copy.num_hidden_layers;

	//null layers
	input_layer = 0;		
	output_layer = 0;
	parray_hidden_layers = 0;
	
	//get input & output pointers
	//its assumed that users want copied
	//nets to share input & output pointer
	//(this is the most obvious behaviour i think)

	input = copy.input;
	output = copy.output;

	//after copying layers
	//we must fix the input pointer to the new net

	//create input layer as copy
	input_layer = meNew NeuronLayer(*copy.input_layer);
	input_layer->input = input;

	//an array of pointers to hidden layers
	parray_hidden_layers = meNew NeuronLayer * [num_hidden_layers];
	//null each one and create it as copy
	for (int i=0;i<num_hidden_layers;i++) 
	{
		parray_hidden_layers[i] = 0;
		parray_hidden_layers[i] = meNew NeuronLayer(*copy.parray_hidden_layers[i]);

		if (i == 0)
			parray_hidden_layers[i]->input = input_layer->output;
		else
		{
			parray_hidden_layers[i]->input = parray_hidden_layers[i - 1]->output;
		}
	}
	//output layer attached to last hidden layer(?? or input layer ??) as copy
	output_layer = meNew NeuronLayer(*copy.output_layer);

	if (num_hidden_layers == 0)
		output_layer->input = input_layer->output;
	else
		output_layer->input = parray_hidden_layers[num_hidden_layers - 1]->output;


	SetResponse(copy.sig_response);
	
	//that should do it
}

NeuralNet::~NeuralNet()
{
	//cleanup
	meDelete input_layer;input_layer = 0;
	meDelete output_layer;output_layer = 0;

	//cleanup an array of pointers to allocated layers

	for (int i=0;i<num_hidden_layers;i++)
	{
		meDelete parray_hidden_layers[i];
		parray_hidden_layers[i] = 0;
	}

	meDelete [] parray_hidden_layers;
	parray_hidden_layers = 0;
}

void NeuralNet::RunNN()
{
	//run each layer
	input_layer->RunLayer();
	for (int i=0;i<num_hidden_layers;i++)
		parray_hidden_layers[i]->RunLayer();

	output_layer->RunLayer();

	//copy output from last layer
	for (i=0;i<num_output_neurons;i++)
		output[i] = output_layer->GetOutput(i);
}

void NeuralNet::SetResponse(float in_sig_response)
{
	sig_response = in_sig_response;
	input_layer->SetResponse(in_sig_response);
	for (int i=0;i<num_hidden_layers;i++)
		parray_hidden_layers[i]->SetResponse(in_sig_response);
	output_layer->SetResponse(in_sig_response);
}

void NeuralNet::SetInputOutput(float *in_finput,float *in_foutput)
{
	if (!in_finput || !in_foutput)
	{ 
		cMyError e;e.AlertError("SetInputOutput(), dont pass null pointers!!");return;
	}
	input = in_finput;output = in_foutput; 
}

void NeuralNet::Randomize()
{
	input_layer->Randomize();
	for (int i=0;i<num_hidden_layers;i++)
	{
		parray_hidden_layers[i]->Randomize();
	}
	output_layer->Randomize();
}

//
//	MatNN::TrainableNeuralNet
//

int TrainableNeuralNet::Mutate(float chance,float extent)
{
	int mutated = 0;

	mutated += input_layer->Mutate(chance,extent);
	for (int i=0;i<num_hidden_layers;i++)
	{
		mutated += parray_hidden_layers[i]->Mutate(chance,extent);
	}
	mutated += output_layer->Mutate(chance,extent);

	return mutated;
}